/*
   Modul: M_DIALS (Headerdatei)

   geschrieben von Thomas Knneth
   Friedrich-Neuper-Str. 2b
   90552 Rthenbach

   Diese Datei ist Bestandteil der C-Version von Magic 4.0 von
   Peter Hellinger und Thomas Knneth und darf nur gem den
   Nutzungsbedingungen von Magic fr C verwendet werden

   Alle Rechte beim Autor
*/

#ifndef _MPDIALS_
#define _MPDIALS_

#ifndef _MAGICTYP_
#include <magic\m_types.h>
#endif

#define MoveBox       17
#define SpecButton    18
#define SpecText      19
#define FrameBox      20
#define Helpbutton    21
#define CircleButton  22
#define TripleButton  23

#define TitleText    127 /* Titetext eines Dialogs */
#define Invisible    126 /* "unsichtbares" Objekt */
#define SliderBox    125 /* Grundobjekt eines Sliders */
#define DropCircle   124
#define DropBox      123 /* Dropbox-Objekt */
#define ListBox      122 /* Listbox-Objekt */
#define SpecBox      121 /* GBox, mit Spezialeffekten */
#define SpecBoxchar  120 /* GBoxchar, Zeichen als Shortkey */
#define TimerObjc    119 /* G_STRING, enhlt das Timerintervall */


/*------------------------------------------------------------------------*
 * Verwaltung der zustzlich bentigten Informationen                     *
 *------------------------------------------------------------------------*/
#define aDefault             0L  /* Wir tun nix                           */
#define aSelect              1L  /* SELECTED-Flag setzen                  */
#define aDeselect            2L  /* SELECTED-Flag lschen                 */
#define aCheck               4L  /* CHECK-Flag setzen                     */
#define aUncheck             8L  /* CHECK-Flag lschen                    */
#define aDisable            16L  /* Disable-Flag setzen                   */
#define aEnable             32L  /* Disable-Flag lschen                  */
#define aTriple1            64L  /* Triple-Status 1 setzen                */
#define aTriple2           128L  /* Triple-Status 2 setzen                */
#define aTriple3           256L  /* Triple-Status 3 setzen                */
#define aSlider            512L  /* Slider-Positon setzen                 */
#define aBell             1024L  /* Glocke ertnen lassen                 */
#define aAsterik          2048L  /* Sternchen in Fenstertitel zeichnen    */
#define aNoAsterik        4096L  /* Sternchen aus Fenstertitel lschen    */
#define aDraw             8192L  /* gemeldetes Objekt neuzeichnen         */
#define aDrawSlider      16384L  /* Slider neu zeichnen                   */
#define aDrawChilds      32768L  /* Objekt + Childs des Objekts zeichnen  */
#define aDrawParent      65536L  /* Parent + Childs des Objekts zeichnen  */
#define aDrawDial       131072L  /* Dialog vollstndig neu zeichnen       */
#define aDrawList       262144L  /* Objektliste neuzeichnen               */
#define aClose          524288L  /* Dialog beenden                        */
#define aUpdateSlider  1048576L
#define aTopDialog     2097152L
#define aCallback	   4194304L
#define aDelEdit       8388608L
#define aChangeEdit   16777216L

typedef  int   tCtrlArray;

typedef enum
{
   dNormal,
   dSpecial,
   dFlydial,
   dModal,
   dNonmodal
} Dialmode;

typedef struct
{
   int         	message; /* Was ist passiert?    */
   OBJECT      	*tree;   /* Objektbaum           */
   int         	objc;    /* Welches Objekt?      */
   int         	mx, my;  /* Mausposition         */
   tKey        	key;     /* Tastencode           */
   char        	asc;     /* Ascii-Code           */
   BITSET      	mstate;
   BITSET      	kstate;
   GRECT       	spec;    /* Special-Parameter    */
   int         	start;   /* Edit-Element         */
   int         	pos;     /* Edit-Position        */
   LONGBITSET  	action;  /* Was ist noch zu tun? */
   tCtrlArray  	*list;   /* Zeiger auf Integer-Array */
   void        	*res;
   /* Folgende Werte werden ab Magic Revision 5 eingetragen */
   int          window;  /* Fensterkennung des Dialogs */
   int          open;    /* > 0 Dialog ist geoeffnet */
   Dialmode     mode;    /* Modus, in dem er geoffnet wurde */
} DialParameter;

typedef DialParameter* PtrDialpara;

typedef void (*DialHandler)(PtrDialpara);

#define dDefault           0L   /* Nix besonderes                  */
#define dPos               1L   /* Dialog positionieren            */
#define dAllkeys           2L   /* Alle Tasten im Editfeld melden  */
#define dMouse             4L   /* Mausbewegung melden             */
#define dLeaveOut          8L   /* abbrechen, wenn ausserhalb      */
#define dButton           16L   /* Ok/Abbruch Mechanik             */
#define dUpArrow          32L   /* Pfeil-Oben in Fenster aufnehmen */
#define dDownArrow        64L   /* Pfeil-Unten aufnehmen           */
#define dVSlider         128L   /* Vertikalen Slider aufnehmen     */
#define dLeftArrow       256L   /* Pfeil-Links aufnehmen           */
#define dRightArrow      512L   /* Pfeil-Rechts aufnehmen          */
#define dHSlider        1024L   /* Horizontalen Slider aufnehmen   */
#define dFuller         2048L   /* Fenster-Fuller aufnehmen        */
#define dInfo           4096L   /* Infozeile aufnehmen             */
#define dSizer          8192L   /* Fenster-Sizer aufnehmen         */
#define dIconifier     16384L
#define dCenter        32768L   /* Dialog zwangsweise zentrieren   */
#define dWintop        65536L
#define dForceWindow  131072L   /* Dialog muss ins Fenster         */
#define dWinKeys      262144L

#define CSCREEN        0
#define CMOUSE         1
#define CPOS           2

typedef enum
{
   TriState1,  /* leere Box    */
   TriState2,  /* angekreuzt   */
   TriState3,  /* grau gefllt */
   TriError    /* Fehler!      */
} TripleState;

typedef enum
{
   Select,
   Check,
   SelectMulti,
   CheckMulti
} ListboxMode;

void OpenDial(OBJECT *tree,Dialmode mode,LONGBITSET flags,
                    int edit,int edpos,int ok,int cancel,int xpos,int ypos,
                    const char *name);
void CloseDial(OBJECT *tree);
void DialInfo(OBJECT *tree,Dialmode *mode,int *win,BOOLEAN *open);
BOOLEAN NewDial(OBJECT *tree,DialHandler handler);
void DisposeDial(OBJECT *tree);
void ObjcExtype(OBJECT *tree,int entry,int extyp);
void GetObjcExtype(OBJECT *tree,int entry,int *extyp,int *typ);
void SetUserkey(OBJECT *tree,int objc,tKey key,BOOLEAN action,BOOLEAN doset);
void ResetUserkeys(OBJECT *tree);
void UserValids(OBJECT *tree,const char *str);
void DialCenter(OBJECT *tree,int flag,int xx,int yy,GRECT *rect);
void DialDraw(OBJECT *tree,int entry,int depth,GRECT *clip);
void DialChange(OBJECT *tree,int entry,int state,
                      GRECT *clip,BOOLEAN doset,BOOLEAN draw);
void DisableDialog(OBJECT *tree);
void EnableDialog(OBJECT *tree);
TripleState GetTriple(OBJECT *tree,int entry);
void SetTriple(OBJECT *tree,int entry,TripleState state);
void GetSlider(OBJECT *tree,int entry,int *spos,int *ssize);
void SetSlider(OBJECT *tree,int entry,int spos,int ssize);
tKey DrawString(GRECT *r,int mode,int col,int bcol,int brast,
				int ftyp,int fid,int fsize,
				const char *s,
				BITSET flags,BOOLEAN center,BOOLEAN middle,BOOLEAN del);
void Rect(GRECT *r,int color,int fill);
void Frame(GRECT *r,int color,int thick);
void Shadow(GRECT *r,int thick);
void Line(int x,int y,int w,int h,int color);
void FreeListbox(OBJECT *tree,int item) ;
BOOLEAN AddEntry(OBJECT *tree,int item,BITSET state,void *data,const char *entry);
BOOLEAN DelEntry(OBJECT *tree,int item,int num);
int FindEntry(OBJECT *tree,int item,int start,char *entry);
void GetEntry(OBJECT *tree,int item,int num,void **data,char *entry);
int ActEntry(OBJECT *tree,int item);
int MaxEntry(OBJECT *tree,int item);
void RedrawListbox(OBJECT *tree,int item,int start);
void SetListboxMode(OBJECT *tree,int item,ListboxMode mode);
int FindSelected(OBJECT *tree,int item,int start);
int FindChecked (OBJECT *tree,int item,int start);
void SelectMatching (OBJECT *tree,int item,int start,const char *entry,int state,BOOLEAN sel,BOOLEAN first);
void SelectAll(OBJECT *tree,int item,int start,int state,BOOLEAN sel);
void SetDropboxElement(OBJECT *tree,int item,int entry,BOOLEAN draw);
void SelectEntry(OBJECT *tree,int item,int entry,int state,BOOLEAN sel);
void DropNext(OBJECT *tree,int item);
BOOLEAN InsertEntry(OBJECT *tree,int item,int pos,BITSET state,void *data,const char *entry);
BOOLEAN ChangeEntry(OBJECT *tree,int item,int pos,BITSET state,void *data,const char *entry);
void EnableDropbox(OBJECT *tree,int item,BOOLEAN draw);
void DisableDropbox(OBJECT *tree,int item,BOOLEAN draw);
void Frame2(GRECT *r, int color1, int color2, int thick);

#define  DisposeDials() CallMagic(1101)

void DropEntry(OBJECT *tree,int item,int dir);
void SelectMatchPtr(OBJECT *tree,int item,int start,void *data,int state,BOOLEAN sel,BOOLEAN first);
int FindExact(OBJECT *tree,int item,int start,char *entry);
#endif
